@@ -64,8 +64,9 @@ typedef struct _GVfsJobDelete          GVfsJobDelete;
 typedef struct _GVfsJobMakeDirectory   GVfsJobMakeDirectory;
 typedef struct _GVfsJobMakeSymlink     GVfsJobMakeSymlink;
 typedef struct _GVfsJobCopy            GVfsJobCopy;
-typedef struct _GVfsJobUpload          GVfsJobUpload;
 typedef struct _GVfsJobMove            GVfsJobMove;
+typedef struct _GVfsJobPush            GVfsJobPush;
+typedef struct _GVfsJobPull            GVfsJobPull;
 typedef struct _GVfsJobSetAttribute    GVfsJobSetAttribute;
 typedef struct _GVfsJobQueryAttributes GVfsJobQueryAttributes;
 typedef struct _GVfsJobCreateMonitor   GVfsJobCreateMonitor;
@@ -297,21 +298,7 @@ struct _GVfsBackendClass
 				 GFileCopyFlags flags,
 				 GFileProgressCallback progress_callback,
 				 gpointer progress_callback_data);
-  void     (*upload)            (GVfsBackend *backend,
-				 GVfsJobUpload *job,
-				 const char *destination,
-				 const char *local_path,
-				 GFileCopyFlags flags,
-				 GFileProgressCallback progress_callback,
-				 gpointer progress_callback_data);
-  gboolean (*try_upload)        (GVfsBackend *backend,
-				 GVfsJobUpload *job,
-				 const char *destination,
-				 const char *local_path,
-				 GFileCopyFlags flags,
-				 GFileProgressCallback progress_callback,
-				 gpointer progress_callback_data);
-  void     (*move)              (GVfsBackend *backend,
+   void     (*move)              (GVfsBackend *backend,
 				 GVfsJobMove *job,
 				 const char *source,
 				 const char *destination,
@@ -325,6 +312,38 @@ struct _GVfsBackendClass
 				 GFileCopyFlags flags,
 				 GFileProgressCallback progress_callback,
 				 gpointer progress_callback_data);
+  void     (*push)             (GVfsBackend *backend,
+                                GVfsJobPush *job,
+                                const char *destination,
+                                const char *local_path,
+                                GFileCopyFlags flags,
+                                gboolean remove_source,
+                                GFileProgressCallback progress_callback,
+                                gpointer progress_callback_data);
+  gboolean (*try_push)         (GVfsBackend *backend,
+                                GVfsJobPush *job,
+                                const char *destination,
+                                const char *local_path,
+                                GFileCopyFlags flags,
+                                gboolean remove_source,
+                                GFileProgressCallback progress_callback,
+                                gpointer progress_callback_data);
+  void     (*pull)             (GVfsBackend *backend,
+                                GVfsJobPull *job,
+                                const char *source,
+                                const char *local_path,
+                                GFileCopyFlags flags,
+                                gboolean remove_source,
+                                GFileProgressCallback progress_callback,
+                                gpointer progress_callback_data);
+  gboolean (*try_pull)         (GVfsBackend *backend,
+                                GVfsJobPull *job,
+                                const char *source,
+                                const char *local_path,
+                                GFileCopyFlags flags,
+                                gboolean remove_source,
+                                GFileProgressCallback progress_callback,
+                                gpointer progress_callback_data);
   void     (*set_attribute)     (GVfsBackend *backend,
 				 GVfsJobSetAttribute *set_attribute,
 				 const char *filename,
@@ -379,6 +398,8 @@ void        g_vfs_backend_set_display_name               (GVfsBackend        *ba
 							  const char         *display_name);
 void        g_vfs_backend_set_stable_name                (GVfsBackend        *backend,
 							  const char         *stable_name);
+void        g_vfs_backend_set_x_content_types            (GVfsBackend        *backend,
+							  char              **x_content_types);
 void        g_vfs_backend_set_icon_name                  (GVfsBackend        *backend,
 							  const char         *icon);
 void        g_vfs_backend_set_prefered_filename_encoding (GVfsBackend        *backend,
@@ -396,6 +417,7 @@ void        g_vfs_backend_unregister_mount               (GVfsBackend        *ba
 const char *g_vfs_backend_get_backend_type               (GVfsBackend        *backend);
 const char *g_vfs_backend_get_display_name               (GVfsBackend        *backend);
 const char *g_vfs_backend_get_stable_name                (GVfsBackend        *backend);
+char      **g_vfs_backend_get_x_content_types            (GVfsBackend        *backend);
 const char *g_vfs_backend_get_icon_name                  (GVfsBackend        *backend);
 GMountSpec *g_vfs_backend_get_mount_spec                 (GVfsBackend        *backend);
 GVfsDaemon *g_vfs_backend_get_daemon                     (GVfsBackend        *backend);
